import { ElementType, ReactElement, RefObject, MutableRefObject } from 'react';
import { OriginClassResolver } from './origin-class-resolvers';
import type { VirtualElement } from '@floating-ui/core';
import type { Options as OffsetOptions } from '@floating-ui/core/src/middleware/offset';
import type { Options as ShiftOptions } from '@floating-ui/core/src/middleware/shift';
import type { Options as FlipOptions } from '@floating-ui/core/src/middleware/flip';
import type { Options as AutoPlacementOptions } from '@floating-ui/core/src/middleware/autoPlacement';
import type { Options as HideOptions } from '@floating-ui/core/src/middleware/hide';
import type { Placement, Strategy, Middleware, DetectOverflowOptions } from '@floating-ui/dom';
import type { Options as AutoUpdateOptions } from '@floating-ui/dom/src/autoUpdate';
interface ArrowState {
    arrowRef: RefObject<HTMLElement>;
    placement: Placement;
    x: number | undefined;
    y: number | undefined;
}
export declare function useArrowContext(component: string): ArrowState;
export interface FloatProps {
    as?: ElementType;
    show?: boolean;
    placement?: Placement;
    strategy?: Strategy;
    offset?: OffsetOptions;
    shift?: boolean | number | Partial<ShiftOptions & DetectOverflowOptions>;
    flip?: boolean | number | Partial<FlipOptions & DetectOverflowOptions>;
    arrow?: boolean | number;
    autoPlacement?: boolean | Partial<AutoPlacementOptions & DetectOverflowOptions>;
    hide?: boolean | Partial<HideOptions & DetectOverflowOptions>;
    autoUpdate?: boolean | Partial<AutoUpdateOptions>;
    zIndex?: number | string;
    enter?: string;
    enterFrom?: string;
    enterTo?: string;
    leave?: string;
    leaveFrom?: string;
    leaveTo?: string;
    originClass?: string | OriginClassResolver;
    tailwindcssOriginClass?: boolean;
    portal?: boolean | string;
    transform?: boolean;
    middleware?: Middleware[] | ((refs: {
        referenceEl: MutableRefObject<Element | VirtualElement | null>;
        floatingEl: MutableRefObject<HTMLElement | null>;
    }) => Middleware[]);
    children: ReactElement[];
    onShow?: () => void;
    onHide?: () => void;
    onUpdate?: () => void;
}
declare function FloatRoot(props: FloatProps): JSX.Element;
interface ArrowRenderProp {
    placement: Placement;
}
declare function Arrow(props: {
    as?: ElementType;
    offset?: number;
    className?: string | ((bag: ArrowRenderProp) => string);
    children?: ReactElement | ((slot: ArrowRenderProp) => ReactElement);
}): JSX.Element;
export declare const Float: typeof FloatRoot & {
    Arrow: typeof Arrow;
};
export {};
